@@ -1,11 +1,11 @@
 from abc import ABC, abstractmethod
 from typing import Dict, Sequence, Generic, TypeVar
 
-from sensai.evaluation.eval_stats.eval_stats_clustering import ClusteringUnsupervisedEvalStats, \
+from .eval_stats.eval_stats_clustering import ClusteringUnsupervisedEvalStats, \
     ClusteringSupervisedEvalStats, ClusterLabelsEvalStats
 from .evaluator import MetricsDictProvider
 from ..clustering import EuclideanClusterer
-from ..util.tracking import timed
+from ..util.profiling import timed
 
 TClusteringEvalStats = TypeVar("TClusteringEvalStats", bound=ClusterLabelsEvalStats)
 
@@ -21,7 +21,7 @@ class ClusteringModelEvaluator(MetricsDictProvider, Generic[TClusteringEvalStats
         :return:
         """
         evalStats = self.evalModel(model, **kwargs)
-        return evalStats.getAll()
+        return evalStats.metricsDict()
 
     @abstractmethod
     def evalModel(self, model: EuclideanClusterer, **kwargs) -> TClusteringEvalStats:
