@@ -1,14 +1,22 @@
+import logging
 from abc import ABC, abstractmethod
-from typing import Tuple, Sequence, Generator, Optional, Union, List, Iterator
+import math
+from typing import Tuple, Sequence, Optional, Union, List, Iterator
 
-from torch.autograd import Variable
-import pandas as pd
 import numpy as np
+import pandas as pd
+import sklearn.preprocessing
 import torch
+from torch.autograd import Variable
 
 from .. import normalisation
 from ..data import DataFrameSplitter, DataFrameSplitterFractional
+from ..data_transformation import DFTSkLearnTransformer
 from ..util.dtype import toFloatArray
+from ..util.pickle import setstate
+
+
+log = logging.getLogger(__name__)
 
 
 def toTensor(d: Union[torch.Tensor, np.ndarray, list], cuda=False):
@@ -51,38 +59,53 @@ class TensorScaler(ABC):
         pass
 
 
-class TensorScalerFromVectorDataScaler(TensorScaler):
-    def __init__(self, vectorDataScaler: normalisation.VectorDataScaler, cuda: bool):
-        self.scale = vectorDataScaler.scale
-        if self.scale is not None:
-            self.scale = torch.from_numpy(vectorDataScaler.scale).float()
-        self.translate = vectorDataScaler.translate
-        if self.translate is not None:
-            self.translate = torch.from_numpy(vectorDataScaler.translate).float()
-        if cuda:
-            self.cuda()
+class TensorScalerCentreAndScale(TensorScaler):
+    def __init__(self, centre: Optional[torch.Tensor] = None, scale: Optional[torch.Tensor] = None):
+        self.centre = centre
+        self.scale = scale
 
     def cuda(self):
         if self.scale is not None:
             self.scale = self.scale.cuda()
-        if self.translate is not None:
-            self.translate = self.translate.cuda()
+        if self.centre is not None:
+            self.centre = self.centre.cuda()
 
     def normalise(self, tensor: torch.Tensor) -> torch.Tensor:
-        if self.translate is not None:
-            tensor -= self.translate
+        if self.centre is not None:
+            tensor -= self.centre
         if self.scale is not None:
-            tensor /= self.scale
+            tensor *= self.scale
         return tensor
 
     def denormalise(self, tensor: torch.Tensor) -> torch.Tensor:
         if self.scale is not None:
-            tensor *= self.scale
-        if self.translate is not None:
-            tensor += self.translate
+            tensor /= self.scale
+        if self.centre is not None:
+            tensor += self.centre
         return tensor
 
 
+class TensorScalerFromVectorDataScaler(TensorScalerCentreAndScale):
+    def __init__(self, vectorDataScaler: normalisation.VectorDataScaler, cuda: bool):
+        if vectorDataScaler.scale is not None:
+            invScale = torch.from_numpy(vectorDataScaler.scale).float()
+            scale = 1.0 / invScale
+        else:
+            scale = None
+        centre = vectorDataScaler.translate
+        if centre is not None:
+            centre = torch.from_numpy(vectorDataScaler.translate).float()
+        super().__init__(centre=centre, scale=scale)
+        if cuda:
+            self.cuda()
+
+    def __setstate__(self, state):
+        if "translate" in state:
+            if state["scale"] is not None:  # old representation where scale is actually inverse scale
+                state["scale"] = 1.0 / state["scale"]
+        setstate(TensorScalerFromVectorDataScaler, self, state, renamedProperties={"translate": "centre"})
+
+
 class TensorScalerIdentity(TensorScaler):
     def cuda(self):
         pass
@@ -94,6 +117,24 @@ class TensorScalerIdentity(TensorScaler):
         return tensor
 
 
+class TensorScalerFromDFTSkLearnTransformer(TensorScalerCentreAndScale):
+    def __init__(self, dft: DFTSkLearnTransformer):
+        trans = dft.sklearnTransformer
+        if isinstance(trans, sklearn.preprocessing.RobustScaler):
+            centre = trans.center_
+            scale = trans.scale_
+            isReciprocalScale = True
+        else:
+            raise ValueError(f"sklearn transformer of type '{trans.__class__}' is unhandled")
+        if centre is not None:
+            centre = torch.from_numpy(centre).float()
+        if scale is not None:
+            scale = torch.from_numpy(scale).float()
+            if isReciprocalScale:
+                scale = 1.0 / scale
+        super().__init__(centre=centre, scale=scale)
+
+
 class Tensoriser(ABC):
     """
     Represents a method for transforming a data frame into one or more tensors to be processed by a neural network model
@@ -127,7 +168,7 @@ class Tensoriser(ABC):
 
 class RuleBasedTensoriser(Tensoriser, ABC):
     """
-    Base class for tensorisers which transforms data frames into tensors based on a predefined set of rules and does not require fitting
+    Base class for tensorisers which transform data frames into tensors based on a predefined set of rules and do not require fitting
     """
     def fit(self, df: pd.DataFrame, model=None):
         pass
@@ -149,7 +190,7 @@ class DataUtil(ABC):
     """Interface for DataUtil classes, which are used to process data for neural networks"""
 
     @abstractmethod
-    def splitInputOutputPairs(self, fractionalSizeOfFirstSet) -> Tuple[Tuple[torch.Tensor, torch.Tensor], Tuple[torch.Tensor, torch.Tensor]]:
+    def splitIntoTensors(self, fractionalSizeOfFirstSet) -> Tuple[Tuple[torch.Tensor, torch.Tensor], Tuple[torch.Tensor, torch.Tensor]]:
         """
         Splits the data set
 
@@ -215,13 +256,16 @@ class VectorDataUtil(DataUtil):
         self.outputTensorScaler = TensorScalerFromVectorDataScaler(self.outputVectorDataScaler, cuda)
         self.dataFrameSplitter = dataFrameSplitter
 
+    def __len__(self):
+        return len(self.inputs)
+
     def getOutputTensorScaler(self):
         return self.outputTensorScaler
 
     def getInputTensorScaler(self):
         return self.inputTensorScaler
 
-    def splitInputOutputPairs(self, fractionalSizeOfFirstSet):
+    def _computeSplitIndices(self, fractionalSizeOfFirstSet):
         splitter = self.dataFrameSplitter
         if splitter is None:
             # By default, we use a simple fractional split without shuffling.
@@ -231,14 +275,24 @@ class VectorDataUtil(DataUtil):
             # the splitter to achieve any desired split).
             splitter = DataFrameSplitterFractional(shuffle=False)
         indices_A, indices_B = splitter.computeSplitIndices(self.inputs, fractionalSizeOfFirstSet)
-        A = self._inputOutputPairs(indices_A)
-        B = self._inputOutputPairs(indices_B)
+        return indices_A, indices_B
+
+    def splitIntoTensors(self, fractionalSizeOfFirstSet):
+        indices_A, indices_B = self._computeSplitIndices(fractionalSizeOfFirstSet)
+        A = self._tensorsForIndices(indices_A)
+        B = self._tensorsForIndices(indices_B)
         return A, B
 
-    def _inputOutputPairs(self, indices):
+    def _dataFramesForIndices(self, indices):
         inputDF = self.inputs.iloc[indices]
         outputDF = self.outputs.iloc[indices]
+        return inputDF, outputDF
 
+    def _tensorsForIndices(self, indices):
+        inputDF, outputDF = self._dataFramesForIndices(indices)
+        return self._tensorsForDataFrames(inputDF, outputDF)
+
+    def _tensorsForDataFrames(self, inputDF, outputDF):
         # apply normalisation (if any)
         if self.inputVectorDataScaler.normalisationMode != normalisation.NormalisationMode.NONE:
             inputDF = pd.DataFrame(self.inputVectorDataScaler.getNormalisedArray(inputDF), columns=inputDF.columns, index=inputDF.index)
@@ -247,6 +301,23 @@ class VectorDataUtil(DataUtil):
 
         return self.inputTensoriser.tensorise(inputDF), self.outputTensoriser.tensorise(outputDF)
 
+    def splitIntoDataSets(self, fractionalSizeOfFirstSet, cuda: bool, tensoriseDynamically=False) -> Tuple["TorchDataSet", "TorchDataSet"]:
+        if not tensoriseDynamically:
+            (xA, yA), (xB, yB) = self.splitIntoTensors(fractionalSizeOfFirstSet)
+            return TorchDataSetFromTensors(xA, yA, cuda), TorchDataSetFromTensors(xB, yB, cuda)
+        else:
+            if self.inputVectorDataScaler.normalisationMode != normalisation.NormalisationMode.NONE or \
+                    self.outputVectorDataScaler.normalisationMode != normalisation.NormalisationMode.NONE:
+                raise Exception("Dynamic tensorisation is not supported when using data scaling")
+            indicesA, indicesB = self._computeSplitIndices(fractionalSizeOfFirstSet)
+            inputA, outputA = self._dataFramesForIndices(indicesA)
+            inputB, outputB = self._dataFramesForIndices(indicesB)
+            dsA = TorchDataSetFromDataFramesDynamicallyTensorised(inputA, outputA, cuda, inputTensoriser=self.inputTensoriser,
+                outputTensoriser=self.outputTensoriser)
+            dsB = TorchDataSetFromDataFramesDynamicallyTensorised(inputB, outputB, cuda, inputTensoriser=self.inputTensoriser,
+                outputTensoriser=self.outputTensoriser)
+            return dsA, dsB
+
     def inputDim(self):
         return self.inputs.shape[1]
 
@@ -379,6 +450,12 @@ class TensorTuple:
         else:
             return self.tuple()
 
+    def concat(self, other: "TensorTuple") -> "TensorTuple":
+        if len(self.tensors) != len(other.tensors):
+            raise ValueError("Tensor tuples are incompatible")
+        tensors = [torch.cat([a, b], dim=0) for a, b in zip(self.tensors, other.tensors)]
+        return TensorTuple(tensors)
+
 
 class TorchDataSetFromTensors(TorchDataSet):
     def __init__(self, x: Union[torch.Tensor, Sequence[torch.Tensor]], y: Optional[torch.Tensor], cuda: bool):
@@ -409,7 +486,15 @@ class TorchDataSetFromTensors(TorchDataSet):
             index = torch.LongTensor(range(length))
         start_idx = 0
         while start_idx < length:
-            end_idx = min(length, start_idx + batch_size)
+            remaining_items = length - start_idx
+            is_second_last_batch = remaining_items <= 2*batch_size and remaining_items > batch_size
+            if is_second_last_batch:
+                # to avoid cases where the last batch is excessively small (1 item in the worst case, where e.g. batch
+                # normalisation would not be applicable), we evenly distribute the items across the last two batches
+                adjusted_batch_size = math.ceil(remaining_items / 2)
+                end_idx = min(length, start_idx + adjusted_batch_size)
+            else:
+                end_idx = min(length, start_idx + batch_size)
             excerpt = index[start_idx:end_idx]
             batch = []
             for tensorTuple in tensorTuples:
@@ -428,27 +513,88 @@ class TorchDataSetFromTensors(TorchDataSet):
                 yield batch[0]
             else:
                 yield tuple(batch)
-            start_idx += batch_size
+            start_idx = end_idx
 
     def size(self):
         return len(self.x)
 
 
-class TorchDataSetFromDataFrames(TorchDataSetFromTensors):
+class TorchDataSetFromDataFramesPreTensorised(TorchDataSetFromTensors):
     def __init__(self, input: pd.DataFrame, output: Optional[pd.DataFrame], cuda: bool,
             inputTensoriser: Optional[Tensoriser] = None, outputTensoriser: Optional[Tensoriser] = None):
         if inputTensoriser is None:
             inputTensoriser = TensoriserDataFrameFloatValuesMatrix()
+        log.debug(f"Applying {inputTensoriser} to data frame of length {len(input)} ...")
         inputTensors = inputTensoriser.tensorise(input)
         if output is not None:
             if outputTensoriser is None:
                 outputTensoriser = TensoriserDataFrameFloatValuesMatrix()
+            log.debug(f"Applying {outputTensoriser} to data frame of length {len(output)} ...")
             outputTensors = outputTensoriser.tensorise(output)
         else:
             outputTensors = None
         super().__init__(inputTensors, outputTensors, cuda)
 
 
+class TorchDataSetFromDataFramesDynamicallyTensorised(TorchDataSet):
+    def __init__(self, input: pd.DataFrame, output: Optional[pd.DataFrame], cuda: bool,
+            inputTensoriser: Optional[Tensoriser] = None, outputTensoriser: Optional[Tensoriser] = None):
+        self.inputDF = input
+        self.outputDF = output
+        self.cuda = cuda
+        if inputTensoriser is None:
+            inputTensoriser = TensoriserDataFrameFloatValuesMatrix()
+        self.inputTensoriser = inputTensoriser
+        if output is not None:
+            if len(input) != len(output):
+                raise ValueError("Lengths of input and output data frames must be equal")
+            if outputTensoriser is None:
+                outputTensoriser = TensoriserDataFrameFloatValuesMatrix()
+        self.outputTensoriser = outputTensoriser
+
+    def size(self) -> Optional[int]:
+        return len(self.inputDF)
+
+    def iterBatches(self, batchSize: int, shuffle: bool = False, inputOnly=False):
+        length = len(self.inputDF)
+        if shuffle:
+            index = torch.randperm(length)
+        else:
+            index = torch.LongTensor(range(length))
+        i = 0
+        while i < length:
+            batchIndices = index[i:i+batchSize]
+            inputTensors = TensorTuple(self.inputTensoriser.tensorise(self.inputDF.iloc[batchIndices]))
+            if self.cuda:
+                inputTensors = inputTensors.cuda()
+            if inputOnly:
+                yield inputTensors.item()
+            else:
+                outputTensors = TensorTuple(self.outputTensoriser.tensorise(self.outputDF.iloc[batchIndices]))
+                if self.cuda:
+                    outputTensors = outputTensors.cuda()
+                yield inputTensors.item(), outputTensors.item()
+            i += batchSize
+
+
+class TorchDataSetFromDataFrames(TorchDataSet):
+    def __init__(self, input: pd.DataFrame, output: Optional[pd.DataFrame], cuda: bool,
+            inputTensoriser: Optional[Tensoriser] = None, outputTensoriser: Optional[Tensoriser] = None,
+            tensoriseDynamically=False):
+        if tensoriseDynamically:
+            self._torchDataSet: TorchDataSet = TorchDataSetFromDataFramesDynamicallyTensorised(input, output, cuda,
+                inputTensoriser=inputTensoriser, outputTensoriser=outputTensoriser)
+        else:
+            self._torchDataSet: TorchDataSet = TorchDataSetFromDataFramesPreTensorised(input, output, cuda,
+                inputTensoriser=inputTensoriser, outputTensoriser=outputTensoriser)
+
+    def iterBatches(self, batchSize: int, shuffle: bool = False, inputOnly=False):
+        yield from self._torchDataSet.iterBatches(batchSize, shuffle=shuffle, inputOnly=inputOnly)
+
+    def size(self) -> Optional[int]:
+        return self._torchDataSet.size()
+
+
 class TorchDataSetProviderFromDataUtil(TorchDataSetProvider):
     def __init__(self, dataUtil: DataUtil, cuda: bool):
         super().__init__(inputTensorScaler=dataUtil.getInputTensorScaler(), outputTensorScaler=dataUtil.getOutputTensorScaler(),
@@ -457,5 +603,23 @@ class TorchDataSetProviderFromDataUtil(TorchDataSetProvider):
         self.cuda = cuda
 
     def provideSplit(self, fractionalSizeOfFirstSet: float) -> Tuple[TorchDataSet, TorchDataSet]:
-        (x1, y1), (x2, y2) = self.dataUtil.splitInputOutputPairs(fractionalSizeOfFirstSet)
+        (x1, y1), (x2, y2) = self.dataUtil.splitIntoTensors(fractionalSizeOfFirstSet)
         return TorchDataSetFromTensors(x1, y1, self.cuda), TorchDataSetFromTensors(x2, y2, self.cuda)
+
+
+class TorchDataSetProviderFromVectorDataUtil(TorchDataSetProvider):
+    def __init__(self, dataUtil: VectorDataUtil, cuda: bool, tensoriseDynamically=False):
+        super().__init__(inputTensorScaler=dataUtil.getInputTensorScaler(), outputTensorScaler=dataUtil.getOutputTensorScaler(),
+            inputDim=dataUtil.inputDim(), modelOutputDim=dataUtil.modelOutputDim())
+        self.dataUtil = dataUtil
+        self.cuda = cuda
+        self.tensoriseDynamically = tensoriseDynamically
+
+    def provideSplit(self, fractionalSizeOfFirstSet: float) -> Tuple[TorchDataSet, TorchDataSet]:
+        return self.dataUtil.splitIntoDataSets(fractionalSizeOfFirstSet, self.cuda, tensoriseDynamically=self.tensoriseDynamically)
+
+
+class TensorTransformer(ABC):
+    @abstractmethod
+    def transform(self, t: torch.Tensor) -> torch.Tensor:
+        pass
\ No newline at end of file
