@@ -1,12 +1,12 @@
 import logging
 import os
-from typing import Sequence, Optional, Tuple
+from typing import Sequence, Optional, Tuple, List
 
 import matplotlib.figure
 from matplotlib import pyplot as plt
 import pandas as pd
 
-from .cache import dumpPickle
+from .pickle import dumpPickle
 
 log = logging.getLogger(__name__)
 
@@ -36,8 +36,7 @@ class ResultWriter:
 
     def path(self, filenameSuffix: str, extensionToAdd=None, validOtherExtensions: Optional[Sequence[str]] = None):
         """
-        :param filenameSuffix: the suffix to add (which may or may not already include the file extension "txt", which
-            will be added if it is not already present)
+        :param filenameSuffix: the suffix to add (which may or may not already include a file extension)
         :param extensionToAdd: if not None, the file extension to add (without the leading ".") unless
             the extension to add or one of the extenions in validExtensions is already present
         :param validOtherExtensions: a sequence of valid other extensions (without the "."), only
@@ -65,6 +64,12 @@ class ResultWriter:
             f.write(content)
         return p
 
+    def writeTextFileLines(self, filenameSuffix, lines: List[str]):
+        p = self.path(filenameSuffix, extensionToAdd="txt")
+        self.log.info(f"Saving text file {p}")
+        writeTextFileLines(lines, p)
+        return p
+
     def writeDataFrameTextFile(self, filenameSuffix, df: pd.DataFrame):
         p = self.path(filenameSuffix, extensionToAdd="df.txt", validOtherExtensions="txt")
         self.log.info(f"Saving data frame text file {p}")
@@ -76,6 +81,7 @@ class ResultWriter:
         p = self.path(filenameSuffix, extensionToAdd="csv")
         self.log.info(f"Saving data frame CSV file {p}")
         df.to_csv(p)
+        return p
 
     def writeFigure(self, filenameSuffix, fig, closeFigure=False):
         """
@@ -97,4 +103,34 @@ class ResultWriter:
 
     def writePickle(self, filenameSuffix, obj):
         p = self.path(filenameSuffix, extensionToAdd="pickle")
+        self.log.info(f"Saving pickle {p}")
         dumpPickle(obj, p)
+        return p
+
+
+def writeTextFileLines(lines: List[str], path):
+    """
+    :param lines: the lines to write (without a trailing newline, which will be added)
+    :param path: the path of the text file to write to
+    """
+    with open(path, "w") as f:
+        for line in lines:
+            f.write(line)
+            f.write("\n")
+
+
+def readTextFileLines(path, strip=True, skipEmpty=True) -> List[str]:
+    """
+    :param path: the path of the text file to read from
+    :param strip: whether to strip each line, removing whitespace/newline characters
+    :param skipEmpty: whether to skip any lines that are empty (after stripping)
+    :return: the list of lines
+    """
+    lines = []
+    with open(path, "r") as f:
+        for line in f.readlines():
+            if strip:
+                line = line.strip()
+            if not skipEmpty or line != "":
+                lines.append(line)
+    return lines
\ No newline at end of file
